#pragma once
#include "../GPU/gpu_types.hpp"
#include "../../math.hpp"

namespace JabyEngine {
    namespace GTE {
        namespace internal {
            template<typename T>
            struct VECTOR {
                T x;
                T y;
                T z;
                T pad;

                static constexpr VECTOR create() {
                    return VECTOR::create(0, 0, 0);
                }

                static constexpr VECTOR create(T x, T y, T z) {
                    return VECTOR{.x = x, .y = y, .z = z, .pad = 0};
                }

                template<typename S>
                static constexpr VECTOR from(const GPU::Position<S>& pos) {
                    return VECTOR::create(static_cast<T>(pos.x), static_cast<T>(pos.y), 0);
                }

                template<typename S>
                constexpr S to() const {
                    return S::create(static_cast<S::PrimitiveType>(this->x), static_cast<S::PrimitiveType>(this->y));
                }
            };
        }
        using VECTOR  = internal::VECTOR<int32_t>;
        using SVECTOR = internal::VECTOR<int16_t>;

        struct ROTMATRIX {
            int16_t matrix[3][3];

            static constexpr ROTMATRIX identity() {
                return ROTMATRIX{.matrix = {
                        {4096, 0, 0},
                        {0, 4096, 0},
                        {0, 0, 4096}
                    }
                };
            } 
            
            static ROTMATRIX rotated(deg_t x = deg_t::zero(), deg_t y = deg_t::zero(), deg_t z = deg_t::zero());
        };

        struct TRANSFERVECTOR : public VECTOR {
            static constexpr TRANSFERVECTOR identity() {
                return TRANSFERVECTOR::translated();
            }

            static constexpr TRANSFERVECTOR translated(int32_t x = 0, int32_t y = 0, int32_t z = 0) {
                return TRANSFERVECTOR{{.x = x, .y = y, .z = z}};
            }
        };
        
        struct MATRIX {
            ROTMATRIX      rotation;
            TRANSFERVECTOR transfer;

            static constexpr MATRIX identity() {
                return MATRIX{.rotation = ROTMATRIX::identity(), .transfer = TRANSFERVECTOR::identity()};
            }

            static constexpr MATRIX translated(int32_t x = 0, int32_t y = 0, int32_t z = 0) {
                return MATRIX{.rotation = ROTMATRIX::identity(), .transfer = TRANSFERVECTOR::translated(x, y, z)};
            }

            static MATRIX rotated(deg_t x = deg_t::zero(), deg_t y = deg_t::zero(), deg_t z = deg_t::zero()) {
                return MATRIX{.rotation = ROTMATRIX::rotated(x, y, z), .transfer = TRANSFERVECTOR::identity()};
            }

            static MATRIX comp(MATRIX new_matrix, const MATRIX& matrix) {
                new_matrix.comp(matrix);
                return new_matrix;
            }

            MATRIX&      comp(const MATRIX& matrix);
            GPU::Vertex& apply_to(GPU::Vertex& vertex)       const;
            GPU::Vertex  apply_to(const GPU::Vertex& vertex) const;
        };
    }
}